//*****************************************************************************
// File Name	: keyin.c
// 
// Title		: I2C UI slave
// Revision		: 1.0
// Notes		: 
// Target MCU	: Atmel AVR series
// Editor Tabs	: 4
// 
// Revision History:
// -----------	-----------	-----------------------
// 09-Sep-2009	Chuck		Created the program
// 
//*****************************************************************************

//----- Include Files ---------------------------------------------------------
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#include "keyin.h"
#define FALSE 0
#define TRUE -1
#define DISABLE 0
#define ENABLE -1


typedef struct key_t_tag {
	uint8_t cmd;
	uint8_t stat;
	uint8_t filt;
} key_t;


/* keyin globals */
volatile static key_t key_buf;


volatile static uint8_t re_buf_cnt;
volatile static uint8_t *re_in_ptr, *re_out_ptr;
volatile static uint8_t re_buffer[RE_BUF_SIZE+1];


volatile static uint8_t re_enable = DISABLE;



//----- Begin Code ------------------------------------------------------------


ISR(TIMER0_OVF_vect)
{
	static uint8_t renc;
	uint8_t a, b;

	a = ((~PIND & 0x0F)<<4) | (~PINB & 0x0F);
	if (a == key_buf.filt) {
		b = key_buf.stat;
		key_buf.stat = a;
		b = (b ^ a) & a;
		if (b) {
			key_buf.cmd = b;
		}
	}
	key_buf.filt = a;

	if (re_enable) {
		// rotary encoder sensing 
		renc = (renc << 2) + (~PINA & ((1<<PA1)|(1<<PA0)));
		renc &= 0x0F;

		switch (renc) {
		case 0xd: a = cRE_LeftTurn;		break;
		case 0x7: a = cRE_RightTurn;	break;
		default:  a = cRE_NoTurn;		break;
		}

		if (a != cRE_NoTurn) {
			*re_in_ptr++ = a;
			if (re_in_ptr >= re_buffer + RE_BUF_SIZE) re_in_ptr = re_buffer; /* Pointer wrapping */
			++re_buf_cnt;
			if(re_buf_cnt > RE_BUF_SIZE) {
				++re_out_ptr;
				if (re_out_ptr >= re_buffer + RE_BUF_SIZE) re_out_ptr = re_buffer; /* Pointer wrapping */
				re_buf_cnt--;
			}
		}


	}

}

// Rotary Encoder input available
void reEnable(void)
{
	re_enable = ENABLE;
	re_in_ptr  = re_out_ptr = re_buffer;
	re_buf_cnt = 0;
}

void reDisable(void)
{
	re_enable = DISABLE;
}

void timer0Init(void)
{
	// start timer0
	TCNT0 = 0;
	TCCR0B |= (1<<CS01)|(1<<CS00);		// set prescaler by 64 1MHz/256/64->16msec
	TIMSK |= (1<<TOIE0);

	sei();
}


void keyInit(void)
{
	DDRB &= ~(1<<PB0);
	DDRB &= ~(1<<PB1);
	DDRB &= ~(1<<PB2);
	DDRB &= ~(1<<PB3);
	DDRD &= ~(1<<PD0);
	DDRD &= ~(1<<PD1);
	DDRD &= ~(1<<PD2);
	DDRD &= ~(1<<PD3);

	timer0Init();
}

void reInit(void)
{
	DDRA &= ~(1<<PA0);	// for Rotary ENC input
	DDRA &= ~(1<<PA1);	// for Rotary ENC input

	timer0Init();
}

uint8_t keyGetCmd(void)
{
	uint8_t c;

	cli();
	c = key_buf.cmd;
	key_buf.cmd = 0;
	sei();

	return c;
}

uint8_t keyGetStat(void)
{
	uint8_t c;

	cli();
	c = key_buf.stat;
	sei();
	return c;
}

uint8_t reGetCmd(void)
{
	uint8_t c;

	if (re_buf_cnt > 0) {
		cli();
		re_buf_cnt--;
		c = *re_out_ptr;				/* get character from buffer */
		if (++re_out_ptr >= re_buffer + RE_BUF_SIZE) re_out_ptr = re_buffer; /* pointer wrapping */
		sei();
		return c;
	} else {
		return cRE_NoTurn;				/* buffer is empty */
	}
}
